import { Actor } from './actor.js';
import { Action } from './action.js';
import { PostMessage } from './message.js';
import { CryptoUtil } from './crypto.js';

export class Member extends Actor {
  constructor(name, scene) {
    super(name, scene);
    this.actions = {
      submitPost: new Action('submit post', scene),
      initiateVote: new Action('initiate vote', scene),
      castVote: new Action('cast vote', scene),
      revealIdentity: new Action('reveal identity', scene),
    };
    this.votes = new Map();
  }

  async initialize() {
    this.reputationKey = await CryptoUtil.generateAsymmetricKey();
    this.reputationPublicKey = await CryptoUtil.exportKey(this.reputationKey.publicKey);
    this.status.set('Initialized');
    return this;
  }

  async submitPost(forumNode, post, stake) {
    // TODO: Include fee
    const postMessage = new PostMessage({ post, stake });
    await postMessage.sign(this.reputationKey);
    this.actions.submitPost.log(this, forumNode, null, { id: post.id });
    // For now, directly call forumNode.receiveMessage();
    await forumNode.receiveMessage(JSON.stringify(postMessage.toJSON()));
  }

  initiateVote(validationPool, options) {
    // For now, directly call validationPool.initiateVote();
    this.actions.initiateVote.log(this, validationPool);
    return validationPool.initiateVote(this.reputationPublicKey, options);
  }

  async castVote(validationPool, voteId, position, stake, lockingTime) {
    const signingKey = await CryptoUtil.generateAsymmetricKey();
    const signingPublicKey = await CryptoUtil.exportKey(signingKey.publicKey)
    this.votes.set(voteId, {signingPublicKey});
    // TODO: encrypt vote
    // TODO: sign message
    this.actions.castVote.log(this, validationPool);
    validationPool.castVote(voteId, signingPublicKey, position, stake, lockingTime);
  }

  async revealIdentity(validationPool, voteId) {
    const {signingPublicKey} = this.votes.get(voteId);
    // TODO: sign message
    this.actions.revealIdentity.log(this, validationPool);
    validationPool.revealIdentity(voteId, signingPublicKey, this.reputationPublicKey);
  }
}
